# rspec/airborne/spaceborne based API tests

## Setup
If you have rvm set up reasonably on your system, the .ruby-XXX files in the apitest directory should create the rvm gemset for you. You will then need to:

```ruby
gem install bundler
bundle install
```

## Environment
| Environment Variable | values | default |
|:------ |:----- |:----- |
| TEST_ENV | dev, local, qa, staging, prod, preprod | dev |
| NAMESPACE | as in other test environments | |                    
| API_KEY   | api key for redshirt account | |
| REDSHIRT_PW | the redshirt account password | |

To run tests against service deployed in EKS cluster, the following steps have to be run:
 - awsrole -u
 - aws-vault exec prod -- ch ecr login
 - kubectl config use-context virginia-1.staging.cloudhealthtech.com
 - awsrole ngstg
 
For information on setting awsrole, and other eks kubectl info, see [here](https://cloudhealthtech.atlassian.net/wiki/spaces/EN/pages/1867710683/Using+Kubectl+with+EKS+clusters+in+Staging+and+Production)

To run these tests locally, cd to the apitest directory.

To run the tests against the service in the preprod env
```
TEST_ENV=preprod EKS_ENV=preprod NAMESPACE=<your_namespace> API_KEY=<api_key> REDSHIRT_PW=<account_password> rspec

```

To run the tests against the service deployed to the dev/staging/prod cluster, specify the env and namespace as such:

```
TEST_ENV=dev EKS_ENV=virginia-1 NAMESPACE=<your_namespace> API_KEY=<api_key> REDSHIRT_PW=<account_password> rspec

```
the api key and password for redshirt is available from the Quality team

to only run one spec file

```
TEST_ENV=staging EKS_ENV=virginia-1 NAMESPACE=<your_namespace> API_KEY=<api_key> REDSHIRT_PW=<account_password> rspec spec/savings_plan_simulation_spec.rb 
```

to only run part of one spec - append a ':123' where 123 is the line number in the describe or it block

to have a failing test perform and output the response using curl, change the appropriate header to dbg_header or set the environment variable ```FORCE_CURL_DEBUG=true```

To run against Production or staging, just pass in 'prod' or 'staging' as the TEST_ENV. The NAMESPACE value can be anything when running against a non dev env

## Setting up new API tests for a service

```ch new service 'savings-plans'```
 will create a new ruby service with an apitest directory structure for you. Once you have your service stood up in the dev environment, you should be able to run the following (from the new service's apitest directory):

```ruby
gem install bundler
bundle install
TEST_ENV=dev rspec spec/service_template_spec.rb
```
This will test that you can get an auth token from local-auth, and that your service can support introspection.

You may need to make some manual adjustments to the `lib/cht_apitest_savings_plans/service.rb` file if your staging and prod deployments use a namespace other than savings-plans. Change the `default_namespace` value to the namespace used. In addition, if the service deployed in that namespace differs from savings-plans, you should modify the `service_name` value.

Once this works, you will probably want to get support for request formatters, response checkers and shortcuts. This is done by introspection on your service, and will automatically change some files to support your service. Run the following:

```ruby
TEST_ENV=dev bin/add_service
```
Some manual adjustment of the lib/helpers/formatter.yml file may be required if your service's schema has responses that sometimes aren't present or return null. You can allow this by appending ```'_or_null'``` to the value specified. For the above example, the clientMutationId returns null unless your request specifies one, so modify the line to contain:

```:clientMutationId: :string_or_null```

Another helper that can get you going with apitests quickly is by running:

```ruby
bin/add_feature -f all
```
This will take any mutation/query that your service supports and attempting to build a spec for that feature that contains a positive and negative test case. The generated specs will still need adjustment to supply legal input values (which introspection can't really give guidance on).

## Failures
### Schema changes
In the event of a schema change, you will get a test failure of the test like:

SavingsPlans introspection - graphql schema changes for savings-plans detects schema changes for savings-plans

First - Alert the QE or Team member responsible for the API tests, and make sure they are aware of the schema change, and create stories to enhance the tests to fix the discrepency between the apitests in the branch/repo, and the schema changes that are part of the PR. This usually involves adding some test cases or validation for new fields or calls.

Second - You can update the supported schema on your branch so that you will no longer see the failure. This would usually be done against the pods running your service for the jenkins PR build that failed the schema change test. From your local system, in the service's apitest directory, run the following 
- run the setup steps to run against EKS as laid out in 'Environment' section above
- change the namespace to the one used for the jenkins run 
- run the following command

```TEST_ENV=staging NAMESPACE=savings-plans-pr-### EKS_ENV=virginia-1 rspec --tag save_schema```

This will change the schema in the supported_schemas directory on your branch, which you can then add to your PR to get the tests to pass.

The tests are not there to make your life harder, but to ensure that we don't have schema changes being deployed without QE being aware that test changes are required.