require 'spec_helper'

module SavingsPlansTest
  shared_examples_for 'api-proxy tests' do | request, variables |
    it "returns all expected objects as per schema via api-proxy for #{request}" do
      wrap_request do
        graphql url(service: ApiProxySavingsPlans.new),
                format_request(request, service: ApiProxySavingsPlans.new),
                variables, headers
        expect_success_graphql
        check(request, service: ApiProxySavingsPlans.new)
      end
      if request == :createSpSavedRecommendations
        delete_variables = {
            recommendationId: json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
        }
        get_api_response(delete_variables, headers, :deleteSpSavedRecommendation)
        expect_success_graphql
      end
    end
  end

  # Skipping below tests until these integration tests run in a post deploy stage
  # Currently, these tests run on branch builds of the service, so there's no utility in running
  # these tests via the api-proy, as the api-proxy would route requests to the deployed version
  # of the service built off master, rather than the version of the service built off
  # the branch under test, and running in the branch pr namespace
  xdescribe 'SavingsPlans routes via api-proxy' do
    include_context 'request metadata'

    request_body = [
      {
        awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
        evaluationPeriod: 'THIRTY_DAYS',
        offeringType:  'PARTIAL_UPFRONT',
        savingsPlanTerm: 'THREE_YEAR'
      }
    ]

    context 'availableAccounts ' do
      variables = {}
      it_should_behave_like 'api-proxy tests', :availableAccounts, variables
    end

    context 'availableBillingAccounts ' do
      variables = {}
      it_should_behave_like 'api-proxy tests', :availableBillingAccounts, variables
    end

    context 'defaultSavingsPlanRecommendations ' do
      variables = {}
      it_should_behave_like 'api-proxy tests', :defaultSavingsPlanRecommendations, variables
    end

    context 'savingsPlanRecommendations ' do
      variables = {
        options: {
          awsBillingAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
              overrideHourlyCommitment: 0.1,
              overrideTotalCoverage: 0.1,
              overrideType: 'DEFAULT'
          }
        }
      }
      it_should_behave_like 'api-proxy tests', :savingsPlanRecommendations, variables
    end

    context 'savingsPlanSimulation ' do
      variables = {
          options: {
              awsBillingAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
              considerExpiringUsage: false,
              evaluationPeriod: 'THIRTY_DAYS',
              hourlyCommitment: 2.0,
              offeringType: 'ALL_UPFRONT',
              term: 'ONE_YEAR',
              type: 'COMPUTE'
          }
      }
      it_should_behave_like 'api-proxy tests', :savingsPlanSimulation, variables
    end

    context 'spSavedRecommendations ' do
      variables = {
          'filters': request_body
      }

      before(:each) {
        create_saved_recs_if_none_exist(request_body)
      }
      it_should_behave_like 'api-proxy tests', :spSavedRecommendations, variables
    end

    context 'spSavedSimulation ' do
      saved_rec_variables = {
          'filters': request_body
      }

      saved_sim_variables = {}

      before(:each) {
        create_saved_recs_if_none_exist(request_body)
        get_api_response(saved_rec_variables, headers, :spSavedRecommendations)
        saved_sim_variables[:recommendationId] = json_body[:data][:spSavedRecommendations][:edges][0][:node][:id]
      }
      it_should_behave_like 'api-proxy tests', :spSavedSimulation, saved_sim_variables
    end

    context 'refreshSpSavedRecommendation ' do
      saved_rec_variables = {
          'filters': request_body
      }

      refresh_variables = {}

      before(:each) {
        create_saved_recs_if_none_exist(request_body)
        get_api_response(saved_rec_variables, headers, :spSavedRecommendations)
        refresh_variables[:recommendationId] = json_body[:data][:spSavedRecommendations][:edges][0][:node][:id]
      }
      it_should_behave_like 'api-proxy tests', :refreshSpSavedRecommendation, refresh_variables
    end

    context 'createSpSavedRecommendations ' do
      future_date = "#{Date.today + 10}T00:00:00Z"
      variables = {
        input: [
          {
             awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_3_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_3[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR'
          }
        ]
      }
      it_should_behave_like 'api-proxy tests', :createSpSavedRecommendations, variables
    end
  end
end
