require 'spec_helper'

module SavingsPlansTest
  account_types = ['linked', 'consolidated', 'unknown', 'standalone']
  account_nodes = []
  linked_accounts = []
  consolidated_accounts = []
  total_count = 0

  describe 'SavingsPlans availableAccounts route' do
    include_context 'request metadata'
    before(:each) {
      if account_nodes.empty?
        variables = {}
        get_api_response(variables, headers, :availableAccounts)
        account_nodes = json_body[:data][:availableAccounts][:edges]
        total_count = json_body[:data][:availableAccounts][:totalCount]
        account_nodes.each { |account|
          expect(account_types).to include(account[:node][:accountType].to_s.downcase)
          if account[:node][:accountType].to_s.downcase == 'linked'
            linked_accounts << account[:node]
          elsif account[:node][:accountType].to_s.downcase == 'consolidated'
            consolidated_accounts << account[:node]
          end
        }
      end
    }

    it 'returns all expected objects as per schema' do
      check(:availableAccounts, service: service)
    end

    it 'totalCount reflects number of account nodes' do
      expect(total_count).to eq(account_nodes.size)
    end

    it 'all account nodes have valid account types' do
      expect(linked_accounts.length).to be > 0
      account_nodes.each { |account|
        expect(account_types).to include(account[:node][:accountType].to_s.downcase)
      }
    end

    it 'returns expected linked account information' do
      expect(linked_accounts.length).to be > 0
      linked_accounts.each { |linked_account|
        expect(linked_account[:ownerId]).not_to be_nil
        expect(linked_account[:payerAccountId]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id])
      }
    end

    it 'returns expected consolidated account information' do
      expect(consolidated_accounts.length).to be > 0
      consolidated_accounts.each { |consolidated_account|
        expect(consolidated_account[:ownerId]).not_to be_nil
        expect(consolidated_account[:id]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id])
        expect(consolidated_account[:payerAccountId]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id])
      }
    end

    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
        }
        graphql url(service: service),
                format_request(
                    :availableAccounts,
                    service: service
                ),
                variables, apitest_headers
        expect_status(403)
        expect(json_body[:data]).to be_nil
        expect(json_body[:error]).to_not be_nil
      end
    end
  end
end
