require 'spec_helper'

module SavingsPlansTest
  describe 'SavingsPlans refreshSpSavedRecommendation: E2E TEST' do
    include_context 'request metadata'

    # common method to abstract verification of refresh request completion
    def verify_quote_is_refreshed(create_quote_variables, refreshed_quote, original_saved_date)
      expect(refreshed_quote[:awsAccountId]).to eq(create_quote_variables[:input][0][:awsAccountId])
      expect(refreshed_quote[:evaluationPeriod]).to eq(create_quote_variables[:input][0][:evaluationPeriod])
      expect(refreshed_quote[:offeringType]).to eq(create_quote_variables[:input][0][:offeringType])
      expect(refreshed_quote[:savingsPlanTerm]).to eq(create_quote_variables[:input][0][:savingsPlanTerm])
      expect(refreshed_quote[:status]).to_not eq('FAILED')
      # quote has an updated saved date post refresh
      expect(Time.parse(refreshed_quote[:savedDate])).to be > original_saved_date

      sp_rec_variables = { 'ids': [ refreshed_quote[:id] ] }
      get_api_response(sp_rec_variables, headers, :spSavedRecommendations)
      saved_refreshed_quote = json_body[:data][:spSavedRecommendations][:edges][0][:node]
      expect(saved_refreshed_quote[:purchaseDate]).to eq(refreshed_quote[:purchaseDate])
      expect(saved_refreshed_quote[:status]).to_not eq('FAILED')
      expect(Time.parse(saved_refreshed_quote[:savedDate])).to be > original_saved_date
      # as the quote being refreshed was just created, existing results expected to get used
      if saved_refreshed_quote[:status] != 'COMPLETED'
        # verify refreshed quote gets to COMPLETED state
        quote_completion_check( refreshed_quote[:id] )
        get_api_response(sp_rec_variables, headers, :spSavedRecommendations)
        expect_success_graphql
        check(:spSavedRecommendations, service: service)
        # verify savedDate is updated post completion
        expect(
          Time.parse(json_body[:data][:spSavedRecommendations][:edges][0][:node][:savedDate])).to be >
            Time.parse(refreshed_quote[:savedDate])
      end
    end

    # The following tests attempt to refresh quotes immediately after they have been created/completed

    it 'refresh quote with no purchase date and hourly commit override (30 Days, partial upfront, 3 year)' do
      hourly_commit_override = 2
      create_quote_variables = {
        input: [
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
            overrideConfiguration: {
              overrideType: 'HOURLY_COMMITMENT',
              overrideHourlyCommitment: hourly_commit_override
            }
          }
        ]
      }
      get_api_response(create_quote_variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)

      create_response_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
      original_saved_date = Time.parse(json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:savedDate])
      # ensure the quote is in a COMPLETED state before being refreshed
      quote_completion_check(create_response_id)

      refresh_variables = { recommendationId: create_response_id }
      get_api_response(refresh_variables, headers, :refreshSpSavedRecommendation)
      expect_success_graphql
      check(:refreshSpSavedRecommendation, service: service)
      refreshed_quote = json_body[:data][:refreshSpSavedRecommendation]
      expect(refreshed_quote[:id]).to eq(create_response_id)
      verify_quote_is_refreshed(create_quote_variables, refreshed_quote, original_saved_date)

      # verify drilldown is available on successful quote creation
      sim_variables = { recommendationId: create_response_id }
      get_api_response(sim_variables, headers, :spSavedSimulation)
      expect_success_graphql
      check(:spSavedSimulation, service: service)
      # verify hourly commitment preserved from original create request
      expect(json_body[:data][:spSavedSimulation][:hourlyCommitment].round(2)).to eq(hourly_commit_override.round(2))

      # delete refreshed request
      get_api_response(refresh_variables, headers, :deleteSpSavedRecommendation)
      expect_success_graphql
      check(:deleteSpSavedRecommendation, service: service)
    end

    it 'refresh quote with purchase date and total coverage override (7 Days, no upfront, one year)' do
      future_date = Date.today + 10
      coverage_override = 80
      create_quote_variables = {
        input: [
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            evaluationPeriod: 'SEVEN_DAYS',
            offeringType:  'NO_UPFRONT',
            savingsPlanTerm: 'ONE_YEAR',
            purchaseDate: "#{future_date}T00:00:00Z",
            overrideConfiguration: {
              overrideTotalCoverage: coverage_override,
              overrideType: 'TOTAL_COVERAGE'
            }
          }
        ]
      }
      get_api_response(create_quote_variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)

      create_response_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
      original_saved_date = Time.parse(json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:savedDate])
      # ensure the quote is in a COMPLETED state before being refreshed
      quote_completion_check(create_response_id)

      refresh_variables = { recommendationId: create_response_id }
      get_api_response(refresh_variables, headers, :refreshSpSavedRecommendation)
      expect_success_graphql
      check(:refreshSpSavedRecommendation, service: service)

      refreshed_quote = json_body[:data][:refreshSpSavedRecommendation]
      expect(refreshed_quote[:id]).to eq(create_response_id)
      # verify originally specified purchase date is persisted in refresh response
      expect(refreshed_quote[:purchaseDate]).to eq("#{future_date}T00:00:00Z")
      verify_quote_is_refreshed(create_quote_variables, refreshed_quote, original_saved_date)

      # verify drilldown is available on successful quote refresh
      sim_variables = { recommendationId: create_response_id }
      get_api_response(sim_variables, headers, :spSavedSimulation)
      expect_success_graphql
      check(:spSavedSimulation, service: service)
      # verify coverage override preserved from original create request
      expect(
          json_body[:data][:spSavedSimulation][:summary][:coverageAfter].round(2)).to eq((coverage_override.to_f / 100).round(2))

      # delete refreshed request
      get_api_response(refresh_variables, headers, :deleteSpSavedRecommendation)
      expect_success_graphql
      check(:deleteSpSavedRecommendation, service: service)
    end
  end

  describe 'SavingsPlans refreshSpSavedRecommendation: Negative tests' do
    include_context 'request metadata'
    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
          'recommendationId': 'recommendationId'
        }
        graphql url(service: service),
          format_request(
            :refreshSpSavedRecommendation,
            service: service
          ),
          variables, apitest_headers
        expect_status(403)
        expect(json_body[:error]).to eq('Forbidden')
      end
    end

    it 'returns expected error for invalid id in authorized request' do
      wrap_request do
        variables = {
          'recommendationId': 'invalid_id'
        }
        graphql url(service: service),
          format_request(
            :refreshSpSavedRecommendation,
            service: service
          ),
          variables, headers
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
      end
    end
  end
end
