require 'spec_helper'

module SavingsPlansTest
  aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

  describe 'savingsPlanRecommendations route introspection based tests with all options:' do
    include_context 'request metadata'
    recommendation_request_options = {
      options: {
        awsBillingAccountId: aws_billing_id,
        considerExpiringUsage: false,
        evaluationPeriod: 'THIRTY_DAYS',
        overrideConfiguration: {
          overrideHourlyCommitment: 0.1,
          overrideTotalCoverage: 0.1,
          overrideType: 'DEFAULT'
        }
      }
    }

    it 'route returns all expected objects as per schema' do
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      expect_success_graphql
      check(:savingsPlanRecommendations, service: service)
    end
  end

  describe 'savingsPlanRecommendations tests with evaluationPeriod "THIRTY_DAYS", no override' do
    recommendation_request_options = {
      options: {
        awsBillingAccountId: aws_billing_id,
        evaluationPeriod: 'THIRTY_DAYS'
      }
    }
    include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

    include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

    include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations
  end

  describe 'savingsPlanRecommendations tests with evaluationPeriod "SEVEN_DAYS", no override' do
    recommendation_request_options = {
      options: {
        awsBillingAccountId: aws_billing_id,
        evaluationPeriod: 'SEVEN_DAYS'
      }
    }
    include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

    include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

    include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations
  end

  describe 'savingsPlanRecommendations tests with overrideType "TOTAL_COVERAGE":' do
    context 'THIRTY_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideTotalCoverage: 90,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }
      include_examples 'standard reservation array tests',recommendation_request_options , :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'overrideConfiguration: TOTAL_COVERAGE tests', 'THIRTY_DAYS', :savingsPlanRecommendations
    end

    context 'SEVEN_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          overrideConfiguration: {
            overrideTotalCoverage: 90,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }
      include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'overrideConfiguration: TOTAL_COVERAGE tests', 'SEVEN_DAYS', :savingsPlanRecommendations
    end
  end

  describe 'savingsPlanRecommendations tests with overrideType "HOURLY_COMMITMENT":' do

    context 'THIRTY_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 5,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'overrideConfiguration: HOURLY_COMMITMENT tests', 'THIRTY_DAYS', :savingsPlanRecommendations
    end

    context 'SEVEN_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 5,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'overrideConfiguration: HOURLY_COMMITMENT tests', 'SEVEN_DAYS', :savingsPlanRecommendations
    end
  end

  # following tests verify that recommendations have highest/lowest recommendations etc,
  # which they wouldn't have if overrideType was HOURLY_COMMITMENT etc, when overrideType is DEFAULT
  # regardless of whether a overrideHourlyCommitment is specified
  describe 'savingsPlanRecommendations: overrideHourlyCommitment should not apply when overrideType is DEFAULT' do
    context 'THIRTY_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 2,
            overrideType: 'DEFAULT'
          }
        }
      }
      include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations
    end

    context 'SEVEN_DAYS:' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 2,
            overrideType: 'DEFAULT'
          }
        }
      }
      include_examples 'standard reservation array tests', recommendation_request_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', recommendation_request_options, :savingsPlanRecommendations
    end
  end

  # the following tests are currently to ensure considerExpiringUsage inclusion does not
  # cause any unexpected regressions (not tests considerExpiringUsage functionality explicitly)
  describe 'savingsPlanRecommendations tests with considerExpiringUsage, THIRTY_DAYS period' do
    context 'considerExpiringUsage: true' do
      ceu_true_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          considerExpiringUsage: true
        }
      }
      include_examples 'standard reservation array tests', ceu_true_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', ceu_true_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', ceu_true_options, :savingsPlanRecommendations
    end

    context 'considerExpiringUsage: false (explicit)' do
      ceu_false_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          considerExpiringUsage: false
        }
      }
      include_examples 'standard reservation array tests', ceu_false_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', ceu_false_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', ceu_false_options, :savingsPlanRecommendations
    end
  end

  describe 'savingsPlanRecommendations tests with considerExpiringUsage, SEVEN_DAYS period' do
    context 'considerExpiringUsag: true:' do
      ceu_true_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          considerExpiringUsage: true
        }
      }
      include_examples 'standard reservation array tests', ceu_true_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', ceu_true_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', ceu_true_options, :savingsPlanRecommendations
    end

    context 'considerExpiringUsage: false (explicit)' do
      ceu_false_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          considerExpiringUsage: false
        }
      }
      include_examples 'standard reservation array tests', ceu_false_options, :savingsPlanRecommendations

      include_examples 'recommendation summary tests', ceu_false_options, :savingsPlanRecommendations

      include_examples 'standard savings plan options node tests', ceu_false_options, :savingsPlanRecommendations
    end
  end

  describe 'For savingsPlanRecommendations route negative tests' do
    include_context 'request metadata'

    # Expecting 500 pending resolution of https://cloudhealthtech.atlassian.net/browse/ENG-22612
    it 'route returns expected error for no auth request' do
      input_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              input_options, apitest_headers
      expect_status(403)
      expect(json_body[:data]).to be_nil
      expect(json_body[:error]).to_not be_nil
    end

    it 'can not access other accounts SP Allocation sources' do
      input_options = {
        options: {
          awsBillingAccountId: INCORRECT_CUSTOMER[:aws_billing_id],
          evaluationPeriod: 'THIRTY_DAYS'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanRecommendations,
                service: service
              ),
              input_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when awsBillingAccountId is missing' do
      input_options = {
        options: {
          evaluationPeriod: 'THIRTY_DAYS'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanRecommendations,
                service: service
              ),
              input_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when evaluationPeriod is missing' do
      input_options = {
        options: {
          awsBillingAccountId: aws_billing_id
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanRecommendations,
                  service: service
              ),
              input_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end
  end

  describe 'TOTAL_COVERAGE tests: Overriding coverage above and below the recommended coverage' do
    include_context 'request metadata'
    context 'SEVEN_DAYS:' do
      include_examples 'Overriding (increase/decrease) default coverage tests', 'SEVEN_DAYS', COVERAGE_OVERRIDE[:increase_total_coverage]

      include_examples 'Overriding (increase/decrease) default coverage tests', 'SEVEN_DAYS', COVERAGE_OVERRIDE[:decrease_total_coverage]
    end

    context 'THIRTY_DAYS:' do
      include_examples 'Overriding (increase/decrease) default coverage tests', 'THIRTY_DAYS', COVERAGE_OVERRIDE[:increase_total_coverage]

      include_examples 'Overriding (increase/decrease) default coverage tests', 'THIRTY_DAYS', COVERAGE_OVERRIDE[:decrease_total_coverage]
    end
  end

  describe 'HOURLY_RATE tests: Overriding hourly commit beyond optimum savings' do
    include_context 'request metadata'
    context 'SEVEN_DAYS:' do
      include_examples 'Override hourly commit beyond optimum savings', 'SEVEN_DAYS'
    end

    context 'THIRTY_DAYS:' do
      include_examples 'Override hourly commit beyond optimum savings', 'THIRTY_DAYS'
    end
  end
end
