require 'spec_helper'

module SavingsPlansTest
  aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

  describe 'savingsPlanSimulation route SP options tests for 30 day period:' do
    context 'ONE_YEAR/ALL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/PARTIAL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/PARTIAL_UPFRONT/considerExpiringUsage:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: true,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/NO_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'NO_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/ALL_UPFRONT, integer hourlyCommitment:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 2,
          offeringType: 'ALL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/PARTIAL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/PARTIAL_UPFRONT/considerExpiringUsage:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: true,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/NO_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'NO_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/PARTIAL_UPFRONT: Very high hourly (over) commit ' do
      expected_overcommit = true
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation, expected_overcommit
    end

    context 'THREE_YEAR/NO_UPFRONT: Very high hourly (over) commit ' do
      expected_overcommit = true
      simulation_options = {
          options: {
              awsBillingAccountId: aws_billing_id,
              considerExpiringUsage: false,
              evaluationPeriod: 'THIRTY_DAYS',
              hourlyCommitment: 3.0,
              offeringType: 'NO_UPFRONT',
              term: 'THREE_YEAR',
              type: 'COMPUTE'
          }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation, expected_overcommit
    end
  end

  describe 'savingsPlanSimulation route SP options tests for 7 day period:' do
    context 'ONE_YEAR/ALL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/PARTIAL_UPFRONT, integer hourlyCommitment:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 3,
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/PARTIAL_UPFRONT/considerExpiringUsage:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: true,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'ONE_YEAR/NO_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'NO_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/ALL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'ALL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/PARTIAL_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/PARTIAL_UPFRONT/considerExpiringUsage:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: true,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'PARTIAL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/NO_UPFRONT:' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 'default_commit',
          offeringType: 'NO_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end

    context 'THREE_YEAR/ALL_UPFRONT: Very high hourly (over) commit' do
      expected_overcommit = true
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'SEVEN_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'THREE_YEAR',
          type: 'COMPUTE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation, expected_overcommit
    end
  end

  describe 'savingsPlanSimulation summary/costs get updated' do
    include_context 'request metadata'

    it 'when hourlyCommitment is updated in multiple requests' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 2.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      less_hourly_commit = json_body[:data][:savingsPlanSimulation]
      simulation_options[:options][:hourlyCommitment]= 4.0
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      more_hourly_commit = json_body[:data][:savingsPlanSimulation]

      expect(more_hourly_commit[:summary][:coverageBefore])
        .to eq(less_hourly_commit[:summary][:coverageBefore]),
            "$2 summary: #{less_hourly_commit[:summary]},  $4 summary: #{more_hourly_commit[:summary]}"
      expect(more_hourly_commit[:summary][:totalUpfrontCost])
        .to be > less_hourly_commit[:summary][:totalUpfrontCost],
            "$2 summary: #{less_hourly_commit[:summary]}, $4 summary: #{more_hourly_commit[:summary]}"
      if less_hourly_commit[:summary][:coverageAfter] < 0.8
        expect(more_hourly_commit[:summary][:coverageAfter])
          .to be > less_hourly_commit[:summary][:coverageAfter],
              "$2 summary: #{less_hourly_commit[:summary]},  $4 summary: #{more_hourly_commit[:summary]}"
        expect(more_hourly_commit[:summary][:globalDiscountRateAfter])
          .not_to eq(less_hourly_commit[:summary][:globalDiscountRateAfter]),
                  "$2 summary: #{less_hourly_commit[:summary]},  $4 summary: #{more_hourly_commit[:summary]}"

        od_cost_lower_commit_sum =
            less_hourly_commit[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }
        existing_od_cost_lower_commit_sum =
            less_hourly_commit[:costData][:existingOnDemandCost].inject(0) { |sum, i| sum + i }
        od_cost_higher_commit_sum =
            more_hourly_commit[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }
        existing_od_cost_higher_commit_sum =
            more_hourly_commit[:costData][:existingOnDemandCost].inject(0) { |sum, i| sum + i }

        expect(existing_od_cost_lower_commit_sum).to eq(existing_od_cost_higher_commit_sum)
        expect(od_cost_higher_commit_sum).to be > od_cost_lower_commit_sum
      end
    end

    it 'when term is updated in multiple requests' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      one_year = json_body[:data][:savingsPlanSimulation]
      simulation_options[:options][:term]= 'THREE_YEAR'
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      three_year = json_body[:data][:savingsPlanSimulation]

      expect(three_year[:summary][:totalUpfrontCost])
        .to be > one_year[:summary][:totalUpfrontCost],
            "1 yr summary: #{one_year[:summary]}, 3 yr summary: #{three_year[:summary]}"

      if one_year[:summary][:coverageAfter] < 0.8
        expect(three_year[:summary][:coverageAfter])
          .to be > one_year[:summary][:coverageAfter],
              "1 yr summary: #{one_year[:summary]}, 3 yr summary: #{three_year[:summary]}"
        expect(three_year[:summary][:globalDiscountRateAfter])
          .to be > one_year[:summary][:globalDiscountRateAfter],
              "1 yr summary: #{one_year[:summary]}, 3 yr summary: #{three_year[:summary]}"

        od_cost_1_year_sum =
            one_year[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }
        od_cost_3_year_sum =
            three_year[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }

        expect(od_cost_3_year_sum).to be > od_cost_1_year_sum
      end
    end

    it 'when offeringType is updated in multiple requests' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      partial_upfront = json_body[:data][:savingsPlanSimulation]
      simulation_options[:options][:offeringType]= 'ALL_UPFRONT'
      get_api_response(simulation_options, headers, :savingsPlanSimulation)
      all_upfront = json_body[:data][:savingsPlanSimulation]

      expect(all_upfront[:summary][:totalUpfrontCost])
        .to be > partial_upfront[:summary][:totalUpfrontCost],
            "1 yr summary: #{partial_upfront[:summary]}, 3 yr summary: #{all_upfront[:summary]}"

      if partial_upfront[:summary][:coverageAfter] < 0.8
        expect(all_upfront[:summary][:coverageAfter])
          .to be > partial_upfront[:summary][:coverageAfter],
              "1 yr summary: #{partial_upfront[:summary]}, 3 yr summary: #{all_upfront[:summary]}"
        expect(all_upfront[:summary][:globalDiscountRateAfter])
          .to be > partial_upfront[:summary][:globalDiscountRateAfter],
              "1 yr summary: #{partial_upfront[:summary]}, 3 yr summary: #{all_upfront[:summary]}"

        od_cost_partial_uf_sum =
            partial_upfront[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }
        existing_od_partial_uf_sum =
            partial_upfront[:costData][:existingOnDemandCost].inject(0) { |sum, i| sum + i }
        od_cost_all_uf_sum =
            all_upfront[:costData][:onDemandCostCovered].inject(0) { |sum, i| sum + i }
        existing_od_cost_all_uf_sum =
            all_upfront[:costData][:existingOnDemandCost].inject(0) { |sum, i| sum + i }

        expect(existing_od_partial_uf_sum.round[2]).to eq(existing_od_cost_all_uf_sum.round[2])
        expect(od_cost_all_uf_sum).to be > od_cost_partial_uf_sum
      end
    end
  end

  describe 'savingsPlanSimulation route negative tests' do
    include_context 'request metadata'

    # Expecting 500 pending resolution of https://cloudhealthtech.atlassian.net/browse/ENG-22612
    it 'returns expected error for no auth request' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, apitest_headers
      expect_status(403)
      expect(json_body[:data]).to be_nil
      expect(json_body[:error]).to_not be_nil
    end

    it 'returns expected error when awsBillingAccountId is missing' do
      simulation_options = {
        options: {
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when evaluationPeriod is missing' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when offeringType is missing' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when hourlyCommitment is missing' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'returns expected error when term is missing' do
      simulation_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end

    it 'can not access other accounts SP Allocation sources' do
      simulation_options = {
        options: {
          awsBillingAccountId: INCORRECT_CUSTOMER[:aws_billing_id],
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
      }
      graphql url(service: service),
              format_request(
                :savingsPlanSimulation,
                service: service
              ),
              simulation_options, headers
      expect_status(200)
      expect(json_body[:data]).to be_nil
      expect(json_body[:errors]).to_not be_nil
    end
  end
end
