module OptionsNodeTests

  RSpec.shared_examples 'standard savings plan options node tests' do |input_options, recommendation_type|
    include_context 'request metadata'
    response_options = {}

    before(:each) {
      if response_options.empty?
        get_api_response(input_options, headers, recommendation_type)
        response_options =
          json_body[:data][recommendation_type][:options]
      end
    }

    context 'savingsPlan Options tests' do
      it 'Billing information is returned for correct customer' do
        account_options = response_options[:awsBillingAccount]
        expect(account_options[:id]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id])
        expect(account_options[:name]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:name] : TEST_CUSTOMER[:name])
        expect(account_options[:ownerId]).to eq(TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:owner_id] : TEST_CUSTOMER[:owner_id])
      end

      it 'expected default evaluationPeriod value returned' do
        if !input_options[:options].nil? && !input_options[:options][:evaluationPeriod].nil?
          expect(response_options[:evaluationPeriod]).
            to eq(input_options[:options][:evaluationPeriod])
        else
          expect(response_options[:evaluationPeriod]).to eq("THIRTY_DAYS")
        end

      end

      it 'expected default considerExpiringUsage value returned' do
        if !input_options[:options].nil? && !input_options[:options][:considerExpiringUsage].nil?
          expect(response_options[:considerExpiringUsage]).
            to eq(input_options[:options][:considerExpiringUsage])
        else
          expect(response_options[:considerExpiringUsage]).to eq(false)
        end
      end

      it 'expected default overrideConfiguration values returned' do
        if !input_options[:options].nil? && !input_options[:options][:overrideConfiguration].nil?
          expect(response_options[:overrideConfiguration][:overrideType]).
            to eq(input_options[:options][:overrideConfiguration][:overrideType])

          if input_options[:options][:overrideConfiguration][:overrideTotalCoverage] == nil
            expect(response_options[:overrideConfiguration][:overrideTotalCoverage]).to eq(0.0)
          else
            expect(response_options[:overrideConfiguration][:overrideTotalCoverage]).
              to eq((input_options[:options][:overrideConfiguration][:overrideTotalCoverage]))
          end

          if input_options[:options][:overrideConfiguration][:overrideHourlyCommitment] == nil
            expect(response_options[:overrideConfiguration][:overrideHourlyCommitment]).to eq(0.0)
          else
            expect(response_options[:overrideConfiguration][:overrideHourlyCommitment]).
              to eq((input_options[:options][:overrideConfiguration][:overrideHourlyCommitment]))
          end
        else
          expect(response_options[:overrideConfiguration]).to eq(nil)
        end
      end
    end
  end
end
