require "rspec/json_expectations"
require 'json'

module RecommendationsNodeTests
  RSpec.shared_examples 'standard reservation array tests' do |variables, recommendation_type|
    include_context 'request metadata'
    graphql_response = {}
    recommendations_array = []
    recommended_hourly_commitments =[]
    hourly_commitment_override = false

    before(:each) {
      if graphql_response.empty?
        get_api_response(variables, headers, recommendation_type)
        graphql_response = json_body
        recommendations_array =
          graphql_response[:data][recommendation_type][:recommendations][:edges]
        recommendations_array.each { |recommendation|
          recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
        }
        if recommendation_type != :defaultSavingsPlanRecommendations &&
            !variables[:options][:overrideConfiguration].nil? &&
            variables[:options][:overrideConfiguration][:overrideType] == 'HOURLY_COMMITMENT'
          hourly_commitment_override = true
        end
      end
    }

    it 'all 6 Saving Plan recommendation types are present' do
      expect(recommendations_array.length).to eq(6)
      expect(recommendations_array).to include_unordered_json(
        [{ node: {savingsPlanConfiguration: {
          offeringType: 'PARTIAL_UPFRONT', term: 'THREE_YEAR'}}},
         { node: { savingsPlanConfiguration: {
           offeringType: 'NO_UPFRONT', term: 'THREE_YEAR'}}},
         { node: { savingsPlanConfiguration: {
           offeringType: 'ALL_UPFRONT', term: 'THREE_YEAR'}}},
         { node: { savingsPlanConfiguration: {
           offeringType: 'NO_UPFRONT', term:'ONE_YEAR'}}},
         { node: { savingsPlanConfiguration: {
           offeringType: 'ALL_UPFRONT', term: 'ONE_YEAR'}}},
         { node: { savingsPlanConfiguration: {
           offeringType: 'PARTIAL_UPFRONT', term: 'ONE_YEAR'}}}])
    end

    context 'highest and lowest commitments:' do
      it 'expected number of lowest and highest commitments' do
        empty_sp_recommendation_attribute_count = 0
        highest_sp_recommendation_attribute_count = 0
        lowest_sp_recommendation_attribute_count = 0

        recommendations_array.each { |recommendation|
          if recommendation[:node][:savingsPlanRecommendationAttributes].empty?
            empty_sp_recommendation_attribute_count += 1
          elsif recommendation[:node][:savingsPlanRecommendationAttributes][0] == 'LOWEST_COMMITMENT'
            lowest_sp_recommendation_attribute_count += 1
          elsif recommendation[:node][:savingsPlanRecommendationAttributes][0] == 'HIGHEST_COMMITMENT'
            highest_sp_recommendation_attribute_count += 1
          end
        }

        if hourly_commitment_override == true
          expect(empty_sp_recommendation_attribute_count).to eq(6)
          expect(lowest_sp_recommendation_attribute_count).to eq(0)
          expect(highest_sp_recommendation_attribute_count).to eq(0)
        else
          expect(empty_sp_recommendation_attribute_count).to eq(4)
          expect(lowest_sp_recommendation_attribute_count).to eq(1)
          expect(highest_sp_recommendation_attribute_count).to eq(1)
        end
      end

      it 'lowest recommendation attribute applied to lowest recommendedHourlyCommitment' do
        lowest_sp_recommendation = 0
        recommendations_array.each { |recommendation|
          recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
          if recommendation[:node][:savingsPlanRecommendationAttributes][0] == 'LOWEST_COMMITMENT'
            lowest_sp_recommendation = recommendation[:node][:recommendedHourlyCommitment]
          end
        }
        expect(recommended_hourly_commitments.min).to be > 0.0
        if hourly_commitment_override == true
          expect(lowest_sp_recommendation).to eq(0.0)
        else
          expect(lowest_sp_recommendation).to eq(recommended_hourly_commitments.min)
        end
      end

      it 'highest recommendation attribute applied to highest recommendedHourlyCommitment' do
        highest_sp_recommendation = 0
        recommendations_array.each { |recommendation|
          recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
          if recommendation[:node][:savingsPlanRecommendationAttributes][0] == 'HIGHEST_COMMITMENT'
            highest_sp_recommendation = recommendation[:node][:recommendedHourlyCommitment]
          end
        }

        if hourly_commitment_override == true
          expect(recommended_hourly_commitments.max).to eq recommended_hourly_commitments.min
        else
          expect(recommended_hourly_commitments.max).to be > recommended_hourly_commitments.min
          expect(highest_sp_recommendation).to eq(recommended_hourly_commitments.max)
        end
      end
    end

    it 'valid expiring SP and RI values are present' do
      expect(graphql_response[:data][recommendation_type][:numberOfExpiringSavingsPlans]).to be >= 0
      expect(graphql_response[:data][recommendation_type][:numberOfExpiringReservedInstances]).to be >= 0
    end
  end

  RSpec.shared_examples 'overrideConfiguration: TOTAL_COVERAGE tests' do |eval_period, recommendation_type|
    aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

    it 'coverageAfter reflects overrideTotalCoverage when < 100%' do
      request_options_90_coverage = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period,
          overrideConfiguration: {
            overrideTotalCoverage: 90,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }

      get_api_response(request_options_90_coverage, headers, :savingsPlanRecommendations)
      override_coverage = get_coverage_per_recommendation(json_body)

      expect(override_coverage['THREE_YEAR/ALL_UPFRONT']).to eq(0.9)
      expect(override_coverage['THREE_YEAR/PARTIAL_UPFRONT']).to eq(0.9)
      expect(override_coverage['THREE_YEAR/NO_UPFRONT']).to eq(0.9)
      expect(override_coverage['ONE_YEAR/ALL_UPFRONT']).to eq(0.9)
      expect(override_coverage['ONE_YEAR/PARTIAL_UPFRONT']).to eq(0.9)
      expect(override_coverage['ONE_YEAR/NO_UPFRONT']).to eq(0.9)
    end

    it 'coverageAfter reflects overrideTotalCoverage when 100%' do
      request_options_90_coverage = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period,
          overrideConfiguration: {
            overrideTotalCoverage: 100,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }

      get_api_response(request_options_90_coverage, headers, :savingsPlanRecommendations)
      override_coverage = get_coverage_per_recommendation(json_body)

      expect(override_coverage['THREE_YEAR/ALL_UPFRONT']).to eq(1.0)
      expect(override_coverage['THREE_YEAR/PARTIAL_UPFRONT']).to eq(1.0)
      expect(override_coverage['THREE_YEAR/NO_UPFRONT']).to eq(1.0)
      expect(override_coverage['ONE_YEAR/ALL_UPFRONT']).to eq(1.0)
      expect(override_coverage['ONE_YEAR/PARTIAL_UPFRONT']).to eq(1.0)
      expect(override_coverage['ONE_YEAR/NO_UPFRONT']).to eq(1.0)
    end

    it 'no recommendations get returned when overrideTotalCoverage < coverageBefore' do
      default_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period
        }
      }
      coverage_before = []

      get_api_response(default_options, headers, :savingsPlanRecommendations)
      default_recommendations = json_body[:data][:savingsPlanRecommendations][:recommendations][:edges]
      default_recommendations.each { |recommendation|
        coverage_before <<
            recommendation[:node][:savingsPlanRecommendationSummary][:coverageBefore]
      }
      # the following ensures the overrideTotalCoverage used is less than coverageBefore
      if coverage_before.min > 1
        lesser_override = coverage_before.min - 0.01
        total_cov_options = {
          options: {
            awsBillingAccountId: aws_billing_id,
            evaluationPeriod: eval_period,
            overrideConfiguration: {
              overrideTotalCoverage: lesser_override.round[2],
              overrideType: 'TOTAL_COVERAGE'
            }
          }
        }
        get_api_response(total_cov_options, headers, :savingsPlanRecommendations)
        recommended_hourly_commitments = []

        json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
          expect(recommendation[:node][:recommendedHourlyCommitment]).to eq(0),
            "unexpected recommendedHourlyCommitment for recommendation: #{recommendation}"
          expect(recommendation[:node][:savingsPlanRecommendationSummary][:totalUpfrontCost]).to eq(0),
            "unexpected totalUpfrontCost for recommendation: #{recommendation}"
          expect(recommendation[:node][:savingsPlanRecommendationSummary][:lifetimeSavings]).to eq(0),
            "unexpected lifetimeSavings for recommendation: #{recommendation}"
          expect(recommendation[:node][:savingsPlanRecommendationSummary][:monthlySavings]).to eq(0),
            "unexpected monthlySavings for recommendation: #{recommendation}"
          expect(recommendation[:node][:savingsPlanRecommendationSummary][:monthlySpend]).to eq(0),
           "unexpected monthlySpend for recommendation: #{recommendation}"
          expect(recommendation[:node][:savingsPlanRecommendationSummary][:globalDiscountRateAfter]).to eq(recommendation[:node][:savingsPlanRecommendationSummary][:globalDiscountRateBefore]),
            "unexpected globalDiscountRateAfter for recommendation: #{recommendation}"
          recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
        }
        # adding the following tests to ensure the above block executed as expected
        expect(recommended_hourly_commitments.min).to eq(0)
        expect(recommended_hourly_commitments.max).to eq(0)
      else
        skip('coverageBefore was <1')
      end

    end

    it 'recommendedHourlyCommitment of 0 when overrideTotalCoverage omitted' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period,
          overrideConfiguration: {
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      recommended_hourly_commitments =[]

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        expect(recommendation[:node][:recommendedHourlyCommitment]).to eq(0.0),
                                                                       "unexpected recommendedHourlyCommitment for recommendation: #{recommendation}"
        recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
      }
      expect(recommended_hourly_commitments.min).to eq(0.0)
      expect(recommended_hourly_commitments.max).to eq(0.0)
    end
  end

  RSpec.shared_examples 'overrideConfiguration: HOURLY_COMMITMENT tests' do |eval_period, recommendation_type|
    aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

    it 'recommendedHourlyCommitment should match overrideHourlyCommitment' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 5,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      recommended_hourly_commitments =[]
      expected_hourly_commit =
        recommendation_request_options[:options][:overrideConfiguration][:overrideHourlyCommitment]

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        expect(recommendation[:node][:recommendedHourlyCommitment]).to eq(expected_hourly_commit),
                                                                       "unexpected recommendedHourlyCommitment for recommendation: #{recommendation}"
        recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
      }
      expect(recommended_hourly_commitments.min).to eq(expected_hourly_commit)
      expect(recommended_hourly_commitments.max).to eq(expected_hourly_commit)
    end

    it 'recommendedHourlyCommitment of 0 when overrideHourlyCommitment ommited' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      recommended_hourly_commitments =[]

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        expect(recommendation[:node][:recommendedHourlyCommitment]).to eq(0.0),
          "unexpected recommendedHourlyCommitment for recommendation: #{recommendation}"
        recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
      }
      expect(recommended_hourly_commitments.min).to eq(0.0)
      expect(recommended_hourly_commitments.max).to eq(0.0)
    end

    it 'overrideTotalCoverage value should have no effect when overrideType is HOURLY_COMMITMENT' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 2,
            overrideTotalCoverage: 90,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      recommended_hourly_commitments =[]

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        expect(recommendation[:node][:recommendedHourlyCommitment]).to eq(2),
          "unexpected recommendedHourlyCommitment for recommendation: #{recommendation}"
        recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
      }
      expect(recommended_hourly_commitments.min).to eq(2)
      expect(recommended_hourly_commitments.max).to eq(2)
    end

    it 'overrideHourlyCommitment should not apply when overrideType is DEFAULT' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: 2,
            overrideType: 'DEFAULT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      recommended_hourly_commitments =[]

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        recommended_hourly_commitments << recommendation[:node][:recommendedHourlyCommitment]
      }
      expect(recommended_hourly_commitments.max).to be > recommended_hourly_commitments.min
    end
  end

  RSpec.shared_examples 'Overriding (increase/decrease) default coverage tests' do |eval_period, action|
    include_context 'request metadata'
    aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

    it "A coverage override that #{action}s from the default recommended coverage, leads to less savings for #{eval_period}" do
      coverage_sum = 0
      base_lifetime_savings = {}
      override_lifetime_savings = {}

      base_recommendation_request = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period
        }
      }

      get_api_response(base_recommendation_request, headers, :savingsPlanRecommendations)
      base_recommendations = json_body[:data][:savingsPlanRecommendations][:recommendations][:edges]
      lowest_coverage = base_recommendations[0][:node][:savingsPlanRecommendationSummary][:coverageAfter]

      base_recommendations.each { |rec|
        coverage_sum = coverage_sum + rec[:node][:savingsPlanRecommendationSummary][:coverageAfter]
        term = rec[:node][:savingsPlanConfiguration][:term]
        offering_type = rec[:node][:savingsPlanConfiguration][:offeringType]
        base_lifetime_savings[ "#{term}/#{offering_type}"] = rec[:node][:savingsPlanRecommendationSummary][:lifetimeSavings].to_f.round(2)
        if lowest_coverage > rec[:node][:savingsPlanRecommendationSummary][:coverageAfter]
          lowest_coverage = rec[:node][:savingsPlanRecommendationSummary][:coverageAfter]
        end
      }
      mean_coverage = coverage_sum / 6

      if action == COVERAGE_OVERRIDE[:decrease_total_coverage]
        # make sure that we expect coverage for all 6 recommendation to be reduced, not just most or some
        # so ensure the reduced coverage rate is less than the lowest recommended coverageAfter value by 50%
        updated_coverage = (lowest_coverage / 2).to_f * 100
      elsif
        updated_coverage = (mean_coverage + (mean_coverage * 0.05).to_f) * 100
      end

      override_recommendation_request = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period,
          overrideConfiguration: {
            overrideTotalCoverage: updated_coverage,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
      }
      get_api_response(override_recommendation_request, headers, :savingsPlanRecommendations)
      override_recommendations = json_body[:data][:savingsPlanRecommendations][:recommendations][:edges]
      override_recommendations.each { |rec|
        term = rec[:node][:savingsPlanConfiguration][:term]
        offering_type = rec[:node][:savingsPlanConfiguration][:offeringType]
        override_lifetime_savings["#{term}/#{offering_type}"] = rec[:node][:savingsPlanRecommendationSummary][:lifetimeSavings].to_f.round(2)
      }

      # expectation is you should always see savings decrease, if you change the coverage from the recommended value
      # accepting equal coverage for increased coverage case, to account for recommended coverage that is already near 100%
      if action == COVERAGE_OVERRIDE[:decrease_total_coverage]
        expect(base_lifetime_savings['THREE_YEAR/ALL_UPFRONT']).to be > override_lifetime_savings['THREE_YEAR/ALL_UPFRONT']
        expect(base_lifetime_savings['THREE_YEAR/PARTIAL_UPFRONT']).to be > override_lifetime_savings['THREE_YEAR/PARTIAL_UPFRONT']
        expect(base_lifetime_savings['THREE_YEAR/NO_UPFRONT']).to be > override_lifetime_savings['THREE_YEAR/NO_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/ALL_UPFRONT']).to be > override_lifetime_savings['ONE_YEAR/ALL_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/PARTIAL_UPFRONT']).to be > override_lifetime_savings['ONE_YEAR/PARTIAL_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/NO_UPFRONT']).to be > override_lifetime_savings['ONE_YEAR/NO_UPFRONT']
      elsif action == COVERAGE_OVERRIDE[:increase_total_coverage]
        expect(base_lifetime_savings['THREE_YEAR/ALL_UPFRONT']).to be >= override_lifetime_savings['THREE_YEAR/ALL_UPFRONT']
        expect(base_lifetime_savings['THREE_YEAR/PARTIAL_UPFRONT']).to be >= override_lifetime_savings['THREE_YEAR/PARTIAL_UPFRONT']
        expect(base_lifetime_savings['THREE_YEAR/NO_UPFRONT']).to be >= override_lifetime_savings['THREE_YEAR/NO_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/ALL_UPFRONT']).to be >= override_lifetime_savings['ONE_YEAR/ALL_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/PARTIAL_UPFRONT']).to be >= override_lifetime_savings['ONE_YEAR/PARTIAL_UPFRONT']
        expect(base_lifetime_savings['ONE_YEAR/NO_UPFRONT']).to be >= override_lifetime_savings['ONE_YEAR/NO_UPFRONT']
      else
        raise Exception.new "Unsupported coverage override option, please specify #{COVERAGE_OVERRIDE}"
      end

    end
  end

  RSpec.shared_examples 'recommendation summary tests' do |variables, recommendation_type|
    include_context 'request metadata'
    graphql_response = {}
    recommendations_array = []
    billing_family_scope = false

    before(:each) {
      if graphql_response.empty?
        complete_rec_found = false
        if recommendation_type == :spSavedRecommendations
          get_api_response(variables, headers, :spSavedRecommendations)
          id = json_body[:data][:spSavedRecommendations][:edges][0][:node][:id]
          # ensure that there's a completed rec present so recommendation summary exists
          json_body[:data][:spSavedRecommendations][:edges].each { |recommendation|
            if recommendation[:node][:status] == 'COMPLETED'
              id = recommendation[:node][:id]
              complete_rec_found = true
            end
          }
          if complete_rec_found == false
            create_variables = {
                input: variables[:filters][0]
            }
            get_api_response(create_variables, headers, :createSpSavedRecommendations)
            id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
            quote_completion_check(id)
          end

          complete_rec_variables = {
            'ids': [
              id
            ]
          }
          get_api_response(complete_rec_variables, headers, recommendation_type)
          graphql_response = json_body
          recommendations_array = graphql_response[:data][recommendation_type][:edges]
        else
          get_api_response(variables, headers, recommendation_type)
          graphql_response = json_body
          recommendations_array = graphql_response[:data][recommendation_type][:recommendations][:edges]
          billing_family_scope = true
        end
      end
    }

    it 'totalUpfrontCost is correct for each recommendation, based on recommendedHourlyCommitment' do
      recommendations_array.each { |recommendation|
        summary_node = recommendation[:node][:savingsPlanRecommendationSummary]
        if recommendation_type == :spSavedRecommendations
          num_of_years = get_sp_term_as_int(recommendation[:node][:savingsPlanTerm])
          offering_type = recommendation[:node][:offeringType]
        else
          num_of_years = get_sp_term_as_int(recommendation[:node][:savingsPlanConfiguration][:term])
          offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
        end

        annual_cost = recommendation[:node][:recommendedHourlyCommitment] * 8760

        if offering_type == 'ALL_UPFRONT'
          expected_total_cost = num_of_years * annual_cost
        elsif  offering_type == 'PARTIAL_UPFRONT'
          expected_total_cost = ((annual_cost / 2) * num_of_years).ceil
        else
          expected_total_cost = 0
        end

        expect(expected_total_cost).not_to be_nil,
          "nil expected_total_cost for: #{recommendation}"
        expect(summary_node[:totalUpfrontCost]).not_to be_nil,
          "nil totalUpfrontCost for: #{recommendation}"
        expect(summary_node[:totalUpfrontCost]).to be >= 0,
          "negative totalUpfrontCost for: #{recommendation}"
        expect(summary_node[:totalUpfrontCost].round()).to eq(expected_total_cost.round()),
          "incorrect totalUpfrontCost, expected: #{expected_total_cost}," +
            "but got: #{summary_node[:totalUpfrontCost]}, for: #{recommendation}"
      }
    end

    it 'monthlySavings corresponds with lifetimeSavings for each recommendation' do
      recommendations_array.each { |recommendation|
        summary_node = recommendation[:node][:savingsPlanRecommendationSummary]
        if recommendation_type == :spSavedRecommendations
          num_of_years = get_sp_term_as_int(recommendation[:node][:savingsPlanTerm])
        else
          num_of_years = get_sp_term_as_int(recommendation[:node][:savingsPlanConfiguration][:term])
        end
        expect(summary_node[:monthlySavings]).not_to be_nil,
          "nil monthlySavings for: #{recommendation}"
        if summary_node[:monthlySavings] > 0
          expected_monthly_savings = summary_node[:lifetimeSavings] * 729.6 / (num_of_years * 8760)
          tolerance_factor = expected_monthly_savings * 0.01
          expect(expected_monthly_savings).not_to be_nil,
            "nil expected_monthly_savings for: #{recommendation}"
          expect(summary_node[:monthlySavings]).to be_within(tolerance_factor).of(expected_monthly_savings),
            "incorrect monthlySavings, expected: #{summary_node[:monthlySavings]}," +
              "to be within 1% of: #{expected_monthly_savings}, for: #{recommendation}"
        end
      }
    end

    it 'asOfDate in Recommendation Summary is > current time and within last 90 days' do
      if billing_family_scope
        current_time = Time.now
        current_date = Date.today
        as_of_date = graphql_response[:data][recommendation_type][:asOfDate]
        expect(Time.parse(as_of_date)).to be < current_time
        expect(Date.parse(as_of_date)).to be > current_date - 90
      else
        skip('asOfDate not returned for single account recommendations')
      end
    end

    it 'coverageAfter in each Recommendation Summary > coverageBefore' do
      recommendations_array.each { |recommendation|
        summary_node = recommendation[:node][:savingsPlanRecommendationSummary]
        expect(summary_node[:coverageBefore]).not_to be_nil,
          "nil coverageBefore for: #{recommendation}"
        expect(summary_node[:coverageBefore]).to be >= 0,
          "negative coverageBefore for: #{recommendation}"
        expect(summary_node[:coverageAfter]).not_to be_nil,
          "nil coverageAfter for: #{recommendation}"
        expect(summary_node[:coverageAfter]).to be > summary_node[:coverageBefore],
          "coverageAfter < coverageBefore for recommendation: #{recommendation}"
      }
    end

    it 'globalDiscountRateAfter in each Recommendation Summary > globalDiscountRateBefore' do
      recommendations_array.each { |recommendation|
        summary_node = recommendation[:node][:savingsPlanRecommendationSummary]
        expect(summary_node[:globalDiscountRateBefore]).not_to be_nil,
          "nil globalDiscountRateBefore for: #{recommendation}"
        if summary_node[:monthlySavings] > 0
          expect(summary_node[:globalDiscountRateBefore]).to be >= 0,
            "negative globalDiscountRateBefore for: #{recommendation}"
          expect(summary_node[:globalDiscountRateAfter]).not_to be_nil,
            "nil globalDiscountRateAfter for: #{recommendation}"
          expect(summary_node[:globalDiscountRateAfter]).to be > summary_node[:globalDiscountRateBefore],
            "globalDiscountRateAfter < globalDiscountRateBefore for recommendation: #{recommendation}"
        end
      }
    end
  end

  # The following tests verify behavior post implementation of ENG-26774
  RSpec.shared_examples 'Override hourly commit beyond optimum savings' do |eval_period|
    include_context 'request metadata'
    # this hourly commit figure is intended to be large enough to
    # gaurentee over 100% coverage for Redshirt customer
    initial_hourly_commit = 5000
    aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

    it 'hourly commitment above optimal savings should result in decreasing savings/discounts' do
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: eval_period,
          overrideConfiguration: {
            overrideHourlyCommitment: initial_hourly_commit,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      initial_response = {}
      increased_commit_response = {}
      not_max_commit = false

      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        term = recommendation[:node][:savingsPlanConfiguration][:term]
        offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
        initial_response["#{term}/#{offering_type}"] = recommendation[:node][:savingsPlanRecommendationSummary]
        if recommendation[:node][:savingsPlanRecommendationSummary][:coverageAfter].round() < 1
          not_max_commit = true
        end
      }

      # if for some reason there is a change in Redshirt usage that causes the hourly commit initially
      # specified to not result in over 100% coverage (very unlikely), we triple the commit and try again
      if not_max_commit == true
        initial_hourly_commit = initial_hourly_commit * 3
        recommendation_request_options = {
          options: {
            awsBillingAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            overrideConfiguration: {
              overrideHourlyCommitment: initial_hourly_commit,
              overrideType: 'HOURLY_COMMITMENT'
            }
          }
        }
        get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
        json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
          term = recommendation[:node][:savingsPlanConfiguration][:term]
          offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
          initial_response["#{term}/#{offering_type}"] = recommendation[:node][:savingsPlanRecommendationSummary]
          if recommendation[:node][:savingsPlanRecommendationSummary][:coverageAfter].round() < 1
            not_max_commit = true
          end
        }
      end

      # if for some reason we still don't have 100% coverage for all quotes (very very unlikely)
      # throw error here to alert that test needs to be updated
      if not_max_commit == true
        raise Exception, 'Test now requires a higher initial hourly commit'
      end

      # increase hourly commit to result in decreasing savings, given that 100% coverage already exists
      recommendation_request_options = {
        options: {
          awsBillingAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          overrideConfiguration: {
            overrideHourlyCommitment: initial_hourly_commit * 3,
            overrideType: 'HOURLY_COMMITMENT'
          }
        }
      }
      get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
      json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |recommendation|
        term = recommendation[:node][:savingsPlanConfiguration][:term]
        offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
        increased_commit_response["#{term}/#{offering_type}"] = recommendation[:node][:savingsPlanRecommendationSummary]
      }

      expect(increased_commit_response["ONE_YEAR/ALL_UPFRONT"][:monthlySavings]).to be <
        initial_response["ONE_YEAR/ALL_UPFRONT"][:monthlySavings]
      expect(increased_commit_response["ONE_YEAR/ALL_UPFRONT"][:lifetimeSavings]).to be <
        initial_response["ONE_YEAR/ALL_UPFRONT"][:lifetimeSavings]
      expect(increased_commit_response["ONE_YEAR/ALL_UPFRONT"][:monthlySavings]).to be <
        initial_response["ONE_YEAR/ALL_UPFRONT"][:effectiveDiscount]
      expect(increased_commit_response["ONE_YEAR/ALL_UPFRONT"][:monthlySavings]).to be <
        initial_response["ONE_YEAR/ALL_UPFRONT"][:globalDiscountRateAfter]
    end
  end
end
