require 'spec_helper'

module SavingsPlansTest
  describe 'SavingsPlans spSavedSimulation: 30 Days' do
    include_context 'request metadata'
    expected_overcommit = false

    context 'spSavedSimulation with no overrides, 3 year, partial upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'PARTIAL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR',
        }
      ]
      simulation_options = {}
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with HOURLY_COMMITMENT override, 1 year, no upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'NO_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR',
        }
      ]
      simulation_options = {
        override: {
          overrideHourlyCommitment: 3.68,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with TOTAL_COVERAGE override, 3 year, all upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'ALL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR',
        }
      ]
      simulation_options = {
          override: {
              'overrideTotalCoverage': 90,
              'overrideType': 'TOTAL_COVERAGE'
          }

      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with HOURLY_COMMITMENT override, first discount bucket' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'NO_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR',
        }
      ]
      simulation_options = {
        override: {
          overrideHourlyCommitment: 0.001,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end
  end

  describe 'SavingsPlans spSavedSimulation: 7 days' do
    include_context 'request metadata'
    expected_overcommit = false

    context 'spSavedSimulation with no overrides, 1 year, all upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          offeringType:  'ALL_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR'
        }
      ]
      simulation_options = {}
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with HOURLY_COMMITMENT override, 3 year, partial upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          offeringType:  'PARTIAL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR'
        }
      ]
      simulation_options = {
        override: {
          overrideHourlyCommitment: 3.68,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with TOTAL_COVERAGE override, 1 year, no upfront' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          offeringType:  'NO_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR'
        }
      ]
      simulation_options = {
        override: {
          overrideTotalCoverage: 90,
          overrideType: 'TOTAL_COVERAGE'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end
  end

  describe 'SavingsPlans spSavedSimulation: Over commit' do
    include_context 'request metadata'
    expected_overcommit = true
    context 'spSavedSimulation with HOURLY_COMMITMENT override, large (over commit): Partial upfront, 7 days, 3 year' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          offeringType:  'PARTIAL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR'
        }
      ]
      simulation_options = {
          override: {
            overrideHourlyCommitment: 3,
            overrideType: 'HOURLY_COMMITMENT'
          }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with HOURLY_COMMITMENT override, large (over commit): All upfront, 30 Days, 1 year' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'ALL_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR'
        }
      ]
      simulation_options = {
        override: {
          overrideHourlyCommitment: 3,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end

    context 'spSavedSimulation with HOURLY_COMMITMENT override, large (over commit): No upfront, 30 Days, 3 year' do
      aws_billing_id = TEST_DATA_ENV == 'preprod' ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'NO_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR'
        }
      ]
      simulation_options = {
        override: {
          overrideHourlyCommitment: 3,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, expected_overcommit, request_body
    end
  end

  describe 'SavingsPlans spSavedSimulation: commit override beyond optimum savings increases monthly spend' do
    include_context 'request metadata'

    context '7 days' do
      include_examples 'Override hourly commit beyond optimum savings in drilldown', 'SEVEN_DAYS'
    end

    context '30 days' do
      include_examples 'Override hourly commit beyond optimum savings in drilldown', 'THIRTY_DAYS'
    end
  end

  describe 'Negative tests' do
    include_context 'request metadata'
    rec_id = ''
    request_body = [
      {
        awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
        evaluationPeriod: 'SEVEN_DAYS',
        offeringType:  'NO_UPFRONT',
        savingsPlanTerm: 'THREE_YEAR',
      }
    ]

    before(:each) {
      create_saved_recs_if_none_exist(request_body)
      variables = {
          'filters': request_body
      }
      get_api_response(variables, headers, :spSavedRecommendations)
      rec_id = json_body[:data][:spSavedRecommendations][:edges][0][:node][:id]

      if json_body[:data][:spSavedRecommendations][:edges][0][:node][:status] != 'COMPLETED'
        create_variables = {
          input: request_body
        }
        get_api_response(create_variables, headers, :createSpSavedRecommendations)
        rec_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
        quote_completion_check(rec_id)
      end
    }

    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
          recommendationId: rec_id
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, apitest_headers
        expect_status(403)
      end
    end

    it 'returns expected error for request without rec id' do
      wrap_request do
        variables = {}
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, headers
        expect(json_body[:data]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for request with invalid rec id' do
      wrap_request do
        variables = {
          recommendationId: 'invalid_id'
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:spSavedSimulation]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
      end
    end

    it 'returns expected error for overrideHourlyCommitment request without HOURLY_COMMITMENT overrideType specified' do
      wrap_request do
        variables = {
          recommendationId: rec_id,
          override: {
            overrideHourlyCommitment: 2
          }
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, headers
        expect(json_body[:data]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for overrideTotalCoverage request without TOTAL_COVERAGE overrideType specified' do
      wrap_request do
        variables = {
          recommendationId: rec_id,
          override: {
            overrideTotalCoverage: 90
          }
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, headers
        expect(json_body[:data]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for overide type mismatch' do
      wrap_request do
        variables = {
          recommendationId: rec_id,
          override: {
            overrideHourlyCommitment: 1,
            overrideType: 'TOTAL_COVERAGE'
          }
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:spSavedSimulation]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end
  end
end
